import * as MC from "@minecraft/server";
import { ActionFormData, ModalFormData } from "@minecraft/server-ui";


//Evento para hacer uso del ITEM
MC.world.events.beforeItemUse.subscribe(({
    item, source
}) => {
    const elemento = item;
    const jugador = source;

    if (elemento.typeId === "bk:tabla_marcadores") {
        form_page1(jugador, jugador.dimension);
    }

});


/**
* @param {MC.Entity} jugador
* @param {{ getEntities: () => Iterable<any> | ArrayLike<any>; }} [dimensiones]
*/
function form_page1(jugador, dimensiones) {

    const entidades = Array.from(dimensiones.getEntities({ type: "bk:marcador" }));
    const marcadores = [];
    let marcadores_cont = 0;

    //Forma solo con botones
    const form_Principal = new ActionFormData();
    form_Principal.title("%bk.ui.Pant_Pri.title");
    form_Principal.button("%bk.ui.Pant_Pri.btn_add");
    form_Principal.button("%bk.ui.Pant_Pri.btn_del");

    for (const entity_add of entidades) {
        const tags = entity_add.getTags();
        for (var i = 0; i < tags.length; i++) {
            marcadores[marcadores_cont] = tags[i];
            form_Principal.button(tags[i]);
            marcadores_cont++;
        }
    }

    form_Principal.show(jugador).then(result => {
        if (result.selection === 0) {
            form_Add(jugador, dimensiones);
        } else if (result.selection === 1) {
            form_Remove(jugador, dimensiones);
        } else if (result.canceled === false) {
            jugador.runCommand("playsound mob.shulker.teleport @a[r=5]");
            jugador.runCommand("effect @s blindness 2 0 true");
            jugador.runCommand(`tp ${jugador.nameTag} @e[tag="${marcadores[(result.selection - 2)]}"]`);
        }
    });
}


/**
* @param {MC.Entity} jugador
* @param {{ getEntities: any; spawnEntity?: any; }} dimensiones
*/
function form_Add(jugador, dimensiones) {

    const entidades = Array.from(dimensiones.getEntities({ type: "bk:marcador" }));
    const marcadores = [];
    let marcadores_cont = 0;

    //Forma con campos de texto
    const form_Agregar = new ModalFormData();
    form_Agregar.title("%bk.ui.Pant_Add.title");
    form_Agregar.textField("%bk.ui.Pant_Add.field1", "", "");

    form_Agregar.show(jugador).then(result => {

        let [tfn] = result.formValues;

        if (tfn.trim().length === 0) {
            jugador.runCommand(`tellraw @s {"rawtext":[{"translate":"%bk.ui.Mensage.msg1"}]}`);
        } else {
            let tfn2 = "";
            //Verifica que el marcador exista
            for (const entity_verf of entidades) {
                const tags = entity_verf.getTags();
                for (var i = 0; i < tags.length; i++) {
                    marcadores[marcadores_cont] = tags[i];
                    marcadores_cont++;
                }
            }

            if (marcadores.length !== 0) {
                for (var i = 0; i < marcadores.length; i++) {
                    if (marcadores[i].includes(tfn)) {
                        tfn2 = tfn.concat("_", i);
                    } else if (tfn2 === "") {
                        tfn2 = tfn;
                    }
                }
                const ubicacion = new MC.Location(Math.floor(jugador.location.x) + 0.5, Math.floor(jugador.location.y), Math.floor(jugador.location.z) + 0.5);
                const entity = dimensiones.spawnEntity('bk:marcador', ubicacion);
                entity.nameTag = tfn2;
                entity.addTag(tfn2 + " - " + jugador.nameTag);
                jugador.runCommand(`tellraw @s {"rawtext":[{"translate":"%bk.ui.Mensage.msg2 ${tfn2}"}]}`);
            } else {
                const ubicacion = new MC.Location(Math.floor(jugador.location.x) + 0.5, Math.floor(jugador.location.y), Math.floor(jugador.location.z) + 0.5);
                const entity = dimensiones.spawnEntity('bk:marcador', ubicacion);
                entity.nameTag = tfn;
                entity.addTag(tfn + " - " + jugador.nameTag);
                jugador.runCommand(`tellraw @s {"rawtext":[{"translate":"%bk.ui.Mensage.msg2 ${tfn}"}]}`);
            }
        }
    });
}


/**
* @param {any} jugador
* @param {{ getEntities: any; }} dimensiones
*/
function form_Remove(jugador, dimensiones) {

    const entidades = Array.from(dimensiones.getEntities({ type: "bk:marcador" }));
    const marcadores = [];
    let marcadores_cont = 0;

    for (const entity_del of entidades) {
        const tags = entity_del.getTags();
        for (var i = 0; i < tags.length; i++) {
            if (tags[i].endsWith(jugador.nameTag)) {
                marcadores[marcadores_cont] = tags[i];
                marcadores_cont++;
            }
        }
    }

    if (marcadores_cont !== 0) {
        //Forma con campos de texto
        const form_Eliminar = new ModalFormData();
        form_Eliminar.title("%bk.ui.Pant_Del.title");
        form_Eliminar.dropdown("%bk.ui.Pant_Pri.title", marcadores);

        form_Eliminar.show(jugador).then(result => {

            let [dropdown] = result.formValues;

            for (const entity_del of entidades) {
                const tags = entity_del.getTags();
                for (var i in tags) {
                    if (tags[i] === marcadores[dropdown]) {
                        entity_del.removeTag(marcadores[dropdown]);
                        entity_del.runCommand(`event entity @s remover_marcador`);
                        jugador.runCommand(`tellraw @s {"rawtext":[{"translate":"%bk.ui.Mensage.msg3 ${marcadores[dropdown]}"}]}`);
                    }
                }
            }

        });
    } else {
        jugador.runCommand(`tellraw @s {"rawtext":[{"translate":"%bk.ui.Mensage.msg4"}]}`);
    }
}